package de.fhw.komponentenarchitekturen.knauf.strutsbasics.actions;

import java.util.Map;

import org.apache.struts2.interceptor.SessionAware;

import com.opensymphony.xwork2.Action;
import com.opensymphony.xwork2.ActionSupport;

import de.fhw.komponentenarchitekturen.knauf.strutsbasics.Historie;
import de.fhw.komponentenarchitekturen.knauf.strutsbasics.Seitenlaengen;

/**
 * Action fr das GeometricModel-Projekt: hier erfolgt das Berechnen der Ausgabe !
 * 
 * Ist eine Subklasse von "ActionSupport" (auch wenn das nicht ntig wre)
 * und implementiert "SessionAware" damit sie Daten in die Session setzen kann.
 * 
 * @author Wolfgang Knauf
 * 
 */
public class GeometricModelAction extends ActionSupport implements SessionAware 
{
  /**Eine Version-UID damit Eclipse nicht meckert.
   */
  private static final long serialVersionUID = 1L;
  
  /**Usereingabe: Kante a */
  private double dblA = 0.0;
  /**Usereingabe: Kante b */
  private double dblB = 0.0;
  /**Usereingabe: Kante c */
  private double dblC = 0.0;
  
  /**Errechnete Oberflche */
  private double dblOberflaeche = 0.0;
  /**Errechnetes Volumen */
  private double dblVolumen = 0.0;
  
  /**Die gesamte Historie. Wird beim ersten Verwenden aus dem Session-Context geladen.*/
  private Historie historieGesamt = null;
  
  /**Die Session zum Zwischenspeichern von Daten.*/
  private Map<String, Object> session;

  /**Setzen der Session.
   * @param session Map mit allen Session-Daten
   */
  @Override
  public void setSession(Map<String, Object> session)
  {
    System.out.println ("setSession !!!");
    this.session = session;
  }
  /**Kante a abrufen.
   * @return Kante a
   */
  public double getA()
  {
    return dblA;
  }

  /**Kante a setzen
   * @param dblA Kante a
   */
  public void setA(double dblA)
  {
    this.dblA = dblA;
  }

  /**Kante b abrufen.
   * @return Kante b
   */
  public double getB()
  {
    return dblB;
  }

  /**Kante b setzen
   * @param dblB Kante b
   */
  public void setB(double dblB)
  {
    this.dblB = dblB;
  }

  /**Kante c abrufen.
   * @return Kante c
   */
  public double getC()
  {
    return dblC;
  }

  /**Kante c setzen
   * @param dblC Kante c
   */
  public void setC(double dblC)
  {
    this.dblC = dblC;
  }
  
  /**Volumen abrufen.
   * 
   * @return Das Volumen
   */
  public double getVolumen()
  {
    return this.dblVolumen;
  }
  
  /**Oberflaeche abrufen.
   * 
   * @return Die Oberflaeche
   */
  public double getOberflaeche()
  {
    return this.dblOberflaeche;
  }

  /**Abrufen der gesamten Historie.
   * 
   * Beim ersten Abrufen wird sie in der Session gesucht !
   * 
   * @return Die Berechnungshistorie
   */
  public Historie getHistorieGesamt()
  {
    //Historie in der Session suchen.
    if (this.session == null)
    {
      throw new NullPointerException("session nicht gesetzt !");
    }
    if (this.session.get("historieGesamt") == null)
    {
      this.session.put("historieGesamt", new Historie() );
    }
    this.historieGesamt = (Historie) this.session.get("historieGesamt");
    return this.historieGesamt;
  }
  
  /**
   * Execute-Methode der Action.
   * 
   * @return Hier immer "success"
   * @throws Exception Jeder auftretende Fehler (hier: nichts erwartetes)
   */
  public String execute() throws Exception
  {
    System.out.println ("Berechnen wurde aufgerufen !");
    
    //Ausrechnen:
    this.dblVolumen = this.dblA * this.dblB * this.dblC;
    this.dblOberflaeche = 2 * (this.dblA * this.dblB) + 2 * (this.dblA * this.dblC) + 2 * (this.dblB * this.dblC);
    
    //Zufgen zur Historie !
    Seitenlaengen seitenlaengeAktuell = new Seitenlaengen();
    seitenlaengeAktuell.setA(this.dblA);
    seitenlaengeAktuell.setB(this.dblB);
    seitenlaengeAktuell.setC(this.dblC);
    
    //Die Historie per Get-Methode abrufen damit sie bei Bedarf erzeugt wird.
    this.getHistorieGesamt().addSeitenlaenge( seitenlaengeAktuell );

    //Rckgabe: "success" (String-Konstante in "Action")
    return Action.SUCCESS;
  }
}
